@@ -3,7 +3,7 @@ package XML::RSSLite;
 use strict;
 use vars qw($VERSION);
 
-$VERSION = 0.10;
+$VERSION = 0.16;
 
 sub import{
   no strict 'refs';
@@ -15,7 +15,7 @@ sub import{
 
 
 sub parseRSS {
-  my ($rr, $cref) = @_;
+  my($rr, $cref, $strip) = @_;
 
   die "$rr is not a hash reference"     unless ref($rr)   eq 'HASH';
   die "$cref is not a scalar reference" unless ref($cref) eq 'SCALAR';
@@ -23,7 +23,7 @@ sub parseRSS {
   # Gotta have some content to parse
   return unless $$cref;
 
-  preprocess($cref);
+  preprocess($cref, $strip);
   {
     _parseRSS($rr, $cref), last if index(${$cref}, '<rss')+1;
     _parseRDF($rr, $cref), last if index(${$cref}, '<rdf:RDF')+1;
@@ -35,12 +35,19 @@ sub parseRSS {
 }
 
 sub preprocess {
-  my $cref = shift;
+  my($cref, $strip) = @_;
   $$cref =~ y/\r\n/\n/s;
-  $$cref =~ y{\n\t ~0-9\-+!@#$%^&*()_=a-zA-Z[]\\;':",./<>?}{ }cs;
-#XXX  $$cref =~ s/&(?!0[a-zA-Z0-9]+|#\d+);/amp/gs;
-#XXX Do we wish to (re)allow escaped HTML?!
-  $$cref =~ s{(?:<|&lt;)/?(?:b|i|h\d|p|center|quote|strong)(?:>|&gt;)}{}gsi;
+
+  if( ! defined($strip) ){
+    $$cref =~ y{\n\t ~0-9\-+!@#$%^&*()_=a-zA-Z[]\\;':",./<>?}{ }cs;
+  }
+  elsif($strip eq '1' ){
+    $$cref =~ s/[^:print:]]/ /g;
+  }
+
+  #XXX $$cref =~ s/&(?!0[a-zA-Z0-9]+|#\d+);/amp/gs;
+  #XXX Do we wish to (re)allow escaped HTML?!
+  $$cref =~ s{(?:<|&lt;)/?(?:b|i|h[1-6]|p|center|quote|strong)(?:>|&gt;)}{}gsi;
 }
 
 sub _parseRSS {
@@ -143,10 +150,10 @@ sub postprocess {
       # See if you can use misplaced url in title for empty links
       elsif( exists($i->{'title'}) ){
 	# The next case would trap this, but try to short-circuit the gathering
-	if ($i->{'title'} =~ /^(?:ht)|ftp:/) {
+	if ($i->{'title'} =~ /^(?:https?|ftp):/) {
 	  $i->{'link'} = $i->{'title'};
 	}
-	elsif ($i->{'title'} =~ /"((?:ht)|ftp.*?)"/) {
+	elsif ($i->{'title'} =~ /"((?:https?|ftp).*?)"/) {
 	  $i->{'link'} = $1;
 	  $i->{'title'} =~ s/<.*?>//;
 	}
@@ -156,15 +163,17 @@ sub postprocess {
       }
     }
     
+    # Trim whitespace
+    $i->{'link'} =~ s/\s+$//;
+    $i->{'link'} =~ s/^\s+//;
+
     # Make sure you've got an http/ftp link
-    if( exists( $i->{'link'}) && $i->{'link'} !~ m{^(http|ftp)://}i) {
+    if( exists( $i->{'link'}) && $i->{'link'} !~ m{^(https?|ftp)://}i) {
       ## Rip link out of anchor tag
-      $i->{'link'} =~ m{a\s+href=("|&quot;)?(.*?)("|>|&quot;|&gt;)?}i;
-      if( $2 ){
-	$i->{'link'} = $2;
-      }
-      elsif( $i->{'link'}  =~ m{[\.#/]}i and $rr->{'link'} =~ m{^http://} ){
-	## Smells like a relative url
+      if( ref($i->{'link'}) && $i->{'link'}->{a}->{href} ){
+	$i->{'link'} = $i->{'link'}->{a}->{href} }
+      ## Smells like a relative url
+      elsif( $i->{'link'}  =~ m{^[#/]} and $rr->{'link'} =~ m{^https?://} ){
 	if (substr($i->{'link'}, 0, 1) ne '/') {
 	  $i->{'link'} = '/' . $i->{'link'};
 	}
@@ -203,13 +212,30 @@ sub parseXML{
       if $comments;
   }
 
+  _parseXML($hash, $xml, $tag);
+
+#  #XXX Context of comment is lost!
+#  #Expose comments if requested
+#  do{ push(@$comments, $_->[1]) for @comments } if ref($comments) eq 'ARRAY';
+  if( $comments ){
+    #Restore comments if requested
+    substr(${$xml}, $_->[0], 0, '<!--'.$_->[1].'-->') for @comments;
+
+    #Expose comments if requested
+    do{ push(@$comments, $_->[1]) for @comments } if ref($comments) eq 'ARRAY';
+  }
+}
+
+sub _parseXML{
+  my($hash, $xml, $tag, $index) = @_;
+  my($begin, $end);
 
   #Find topTag and set pos to start matching from there
   ${$xml} =~ /<$tag(?:>|\s)/g;
   ($begin, $end) = (0, pos(${$xml})||0);
 
   #Match either <foo></foo> or <bar />, optional attributes, stash tag name
-  while( ${$xml} =~ m%<([^\s>]+)(?:\s+[^>]*?)?(?:/|>.*?</\1)>%sg ){	 
+  while( ${$xml} =~ m%<([^\s>]+)(?:\s+[^>]*?)?(?:/|>.*?</\1)>%sg ){
 
     #Save the tag name, we'll need it
     $tag = $1 || $2;
@@ -221,57 +247,52 @@ sub parseXML{
     my $str = substr(${$xml}, $begin, $end-$begin);
     
     #Extract the actual attributes and contents of the tag
-#   $str =~ m%<$tag\s*([^>]*?)?>(.*?)</$tag>%s ||
-    $str =~ s%^.*?<$tag\s*([^>]*?)?>(.*?)</$tag>%<$tag>$2</$tag>%s ||
-      $str =~ m%<$tag\s*([^>]*?)?\s*/>%;
+   $str =~ m%<\Q$tag\E\s*([^>]*?)?>(.*?)</\Q$tag\E>%s ||
+#XXX pointed out by hv
+#    $str =~ s%^.*?<$tag\s*([^>]*?)?>(.*?)</$tag>%<$tag>$2</$tag>%s ||
+      $str =~ m%<\Q$tag\E\s*([^>]*?)?\s*/>%;
     my($attr, $content) = ($1, $2);
 
     #Did we get attributes? clean them up and chuck them in a hash.
     if( $attr ){
       ($_, $attr) = ($attr, {});
-      $attr->{$1} = $3 while m/([^\s=]+)\s*=\s*(['"])(.*?)\2/g;
+      $attr->{$1} = $3 while m/([^\s=]+)\s*=\s*(['"]?)([^\2>]*?)(?:\2|$)/g;
     }
 
+    my $inhash;
     #Recurse if contents has more tags, replace contents with reference we get
     if( $content && index($content, '<') > -1 ){
-      parseXML($content={}, \$str, $tag, 0);
+      _parseXML($inhash={}, \$str, $tag);
       #Was there any data in the contents? We should extract that...
-#      if( $str =~ />[^><\s]+</ || $str =~ />(?:[^><\s]+\s+)+</ ){
       if( $str =~ />[^><]+</ ){
-	#The odd RE above \S+\s+ shortcircuits unnecessary entry
-
-	my $length = length($str);
-	my $taglen = length($tag)+2;
-	$str= substr($str, $taglen, $length-1-2*$taglen);
+	#The odd RE above shortcircuits unnecessary entry
 
 	#Clean whitespace between tags
 	#$str =~ s%(?<=>)?\s*(?=<)%%g; #XXX ~same speed, wacko warning
 	#$str =~ s%(>?)\s*<%$1<%g;
+#XXX    #$str =~ s%(?:^|(?<=>))\s*(?:(?=<)|\z)%%g
 
-#	$str =~ s%<$_\s*(?:[^>]*?)?(?:/|>.*?</$_)>%%sg for keys %{$content};
-	my $qr = qr{@{[join('|', keys %{$content})]}};
-	$str =~ s%<($qr)\s*(?:[^>]*?)?(?:/|>.*?</\1)>%%sg;
+	my $qr = qr{@{[join('|', map { quotemeta } keys %{$inhash})]}};
+	$content =~ s%<($qr)\s*(?:[^>]*?)?(?:/|>.*?</\1)>%%sg;
 
-	$content->{'<>'} = $str;#XXX if $str;
+	$inhash->{'<>'} = $content if $content =~ /\S/;
       }
     }
 
-    my($inhash);
-    if( ref($content) ){
-      #We have attributes? Then we should save them.
-      $inhash = $attr || {};
-
-      #Contents too? Save them as well.
-      if( $content ){
-	for( keys %{$content} ){
+    if( ref($inhash) ){
+      #We have attributes? Then we should merge them.
+      if( ref($attr) ){
+	for( keys %{$attr} ){
 	  $inhash->{$_} = exists($inhash->{$_})   ?
 	    (ref($inhash->{$_})  eq 'ARRAY'       ?
-	     [@{$inhash->{$_}}, $content->{$_}]   :
-	     [  $inhash->{$_},  $content->{$_}] ) : $content->{$_};
+	     [@{$inhash->{$_}}, $attr->{$_}]   :
+	     [  $inhash->{$_},  $attr->{$_}] ) : $attr->{$_};
 	}
-	
       }
     }
+    elsif( ref($attr) ){
+      $inhash = $attr;
+    }
     else{
       #Otherwise save our content
       $inhash = $content;
@@ -282,14 +303,6 @@ sub parseXML{
 	[@{$hash->{$tag}}, $inhash]       :
 	[  $hash->{$tag},  $inhash]  )    : $inhash;
   }
-
-  if( $comments ){
-    #Restore comments if requested
-    substr(${$xml}, $_->[0], 0, '<!--'.$_->[1].'-->') for @comments;
-
-    #Expose comments if requested
-    do{ push(@$comments, $_->[1]) for @comments } if ref($comments) eq 'ARRAY';
-  }
 }
 
 1;
@@ -305,8 +318,6 @@ XML::RSSLite - lightweight, "relaxed" RSS (and XML-ish) parser
 
   use XML::RSSLite;
 
-  . . .
-
   parseRSS(\%result, \$content);
 
   print "=== Channel ===\n",
@@ -314,7 +325,7 @@ XML::RSSLite - lightweight, "relaxed" RSS (and XML-ish) parser
         "Desc:  $result{'description'}\n",
         "Link:  $result{'link'}\n\n";
 
-  foreach $item (@{$result{'item'}}) {
+  foreach $item (@{$result{'items'}}) {
   print "  --- Item ---\n",
         "  Title: $item->{'title'}\n",
         "  Desc:  $item->{'description'}\n",
@@ -340,7 +351,9 @@ output for best results. The munging includes:
 
 =item Remove html tags to leave plain text
 
-=item Remove characters other than 0-9~!@#$%^&*()-+=a-zA-Z[];',.:"<>?\s
+=item Remove leading whitespace from URIs
+
+=item By defaul strips characters except 0-9~!@#$%^&*()-+=a-zA-Z[];',.:"<>?\s
 
 =item Use <url> tags when <link> is empty
 
@@ -348,9 +361,9 @@ output for best results. The munging includes:
 
 =item Exract links from <a href=...> if required   
 
-=item Limit links to ftp and http
+=item Limit links to ftp and http(s)
 
-=item Join relative urls to the site base
+=item Join relative item urls (beginning with / or #) to the site base
 
 =back
 
@@ -358,18 +371,37 @@ output for best results. The munging includes:
 
 =over
 
-=item parseRSS($outHashRef, $inScalarRef)
+=item parseRSS($outHashRef, $inScalarRef, [$strip])
 
-I<$inScalarRef> is a reference to a scalar containing the document to be
-parsed, the contents will effectively be destroyed. I<$outHashRef> is a
-reference to the hash within which to store the parsed content.
+=over
 
+=item inScalarRef - required
 
-XXX
-=item usableXML($inScalarRef)
+Reference to a scalar containing the document to be parsed. NOTE: The
+contents will effectively be destroyed. Make a deep copy first if you care.
 
-Test whether or not B<XML::RSSLite> understands the content of the referenced
-document.
+=item outHashRef - required
+
+Reference to the hash within which to store the parsed content.
+
+=item strip - optional
+
+An expression indicating the level of winnowing to be performed on the
+characters permitted in the results.
+
+=over
+
+=item 1 strip non-printable characters
+
+=item 0 no characters are removed
+
+=item undefined (Default) strip everything but:
+
+0-9~!@#$%^&*()-+= a-zA-Z[];',.:"<>?\t\n
+
+=back
+
+=back
 
 =back
 
@@ -379,29 +411,27 @@ document.
 
 =item parseXML(\%parsedTree, \$parseThis, 'topTag', $comments);
 
-=head1 DESCRIPTION
-
 =over
 
 =item parsedTree - required
 
-reference to hash to store the parsed document within
+Reference to hash to store the parsed document within.
 
 =item parseThis  - required
 
-reference to scalar containing the document to parse
+Reference to scalar containing the document to parse.
 
 =item topTag     - optional
 
-tag to consider the root node, leaving this undefined is not recommended.
+Tag to consider the root node, leaving this undefined is not recommended.
 
 =item comments   - optional
 
 =over
 
-=item false will remove contents from
+=item false will remove contents from parseThis
 
-=item true will not remove comments
+=item true will not remove comments from parseThis
 
 =item array reference is true, comments are stored here
 
@@ -409,6 +439,8 @@ tag to consider the root node, leaving this undefined is not recommended.
 
 =back
 
+=back
+
 =head2 CAVEATS
 
 This is not a conforming parser. It does not handle the following
@@ -441,7 +473,7 @@ It's non-validating, without a DTD the following cannot be properly addressed
 
 =item namespaces
 
-This might arriving in the next release.
+This may or may not be arriving in some future release.
 
 =back
 
@@ -460,7 +492,7 @@ Scott Thomason <scott@thomasons.org>
 
 =head1 LICENSE
 
-Portions Copyright (c) 2002 Jerrad Pierce, (c) 2000 Scott Thomason.
+Portions Copyright (c) 2002,2003,2009 Jerrad Pierce, (c) 2000 Scott Thomason.
 All rights reserved. This program is free software; you can redistribute it 
 and/or modify it under the same terms as Perl itself.
 
